'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});
exports.addPrefixedProperties = addPrefixedProperties;
exports.resolveHack = resolveHack;
exports.caplitalizeString = caplitalizeString;
exports.generatePrefixedProperty = generatePrefixedProperty;
exports.isPrefixProperty = isPrefixProperty;
exports.generateRequiredProperties = generateRequiredProperties;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _paramCase = require('param-case');

var _paramCase2 = _interopRequireDefault(_paramCase);

var _utilsGetBrowserInfo = require('./utils/getBrowserInfo');

var _utilsGetBrowserInfo2 = _interopRequireDefault(_utilsGetBrowserInfo);

var _data = require('./data');

var _data2 = _interopRequireDefault(_data);

var _hacks = require('./hacks');

var _hacks2 = _interopRequireDefault(_hacks);

var info = undefined;
var requiredProperties = [];
var requiredHacks = [];
var ua = typeof navigator !== 'undefined' ? navigator.userAgent : undefined;
var generated = false;

exports['default'] = {
	/**
  * Sets the used userAgent
  * @param {string} userAgent - a valid userAgent string
  */
	setUserAgent: function setUserAgent(userAgent) {
		ua = userAgent;
		generateRequiredProperties();
	},

	/**
  * Returns the currently used userAgent
  */
	getUserAgent: function getUserAgent() {
		return ua;
	},

	/**
  * Processes an object of styles using userAgent specific 
  * @param {Object} styles - Styles object that gets prefixed
  * @param {Boolean} hacks - If hacks should be used to resolve browser differences
  */
	process: function process(styles) {
		if (!generated) {
			generateRequiredProperties();
		}
		if (requiredProperties.length > 0) {
			addPrefixedProperties(styles);
		}
		return styles;
	}
};

/**
 * Adds prefixed properties to a style object
 * @param {Object} styles - Style object that gets prefixed properties added
 */

function addPrefixedProperties(styles) {
	var property = undefined;

	var _loop = function () {
		var value = styles[property];

		if (value instanceof Object) {
			//recursively loop through nested style objects
			addPrefixedProperties(value);
		} else {
			//add prefixes if needed
			if (isPrefixProperty(property)) {
				styles[generatePrefixedProperty(property)] = value;
			}

			//resolve hacks
			requiredHacks.forEach(function (hack) {
				resolveHack(hack, styles, property, value);
			});
		}
	};

	for (property in styles) {
		_loop();
	}
	return styles;
}

function getPrefixedValue(property, oldValue, newValue) {
	if (newValue) {
		return value.replace(oldValue, info.prefix.css + newValue, 'g') + ';' + (0, _paramCase2['default'])(property) + ':' + oldValue;
	} else {
		return info.prefix.css + oldValue + ';' + (0, _paramCase2['default'])(property) + ':' + oldValue;
	}
}

/**
 * Resolves browser issues using some hacks
 * @param {Object} hackData - contains a condition and properties/values that need to be corrected
 * @param {Object} styles - a style object
 * @param {string} property - property that gets corrected
 * @param {any} value - property value
 */

function resolveHack(hackData, styles, property, value) {

	//prefix ordinary values
	if (hackData.prefixValue) {

		var values = hackData.prefixValue[property];
		if (values) {
			if (hackData.containValue) {
				values.forEach(function (val) {
					if (value.indexOf(val) > -1) {
						styles[property] = getPrefixedValue(property, value, val);
					}
				});
			} else {
				styles[property] = getPrefixedValue(property, value);
			}
		}
	}

	//resolve property issues
	if (hackData.alternativeProperty) {

		var oldProperty = hackData.alternativeProperty[property];
		if (oldProperty) {
			styles[oldProperty] = value;
		}
	}

	//resolve alternative values
	if (hackData.alternativeValue) {

		var oldValue = hackData.alternativeValue[property];
		if (oldValue && oldValue[value]) {
			styles[property] = oldValue[value] + ';' + (0, _paramCase2['default'])(property) + ':' + value;
		}
	}
}

/**
 * Capitalizes first letter of a string
 * @param {String} str - str to caplitalize
 */

function caplitalizeString(str) {
	return str.charAt(0).toUpperCase() + str.slice(1);
}

/**
 * Returns a prefixed style property
 * @param {String} property - a style property in camelCase
 * @param {String} prefix - evaluated vendor prefix that will be added
 */

function generatePrefixedProperty(property) {
	return info.prefix.inline + caplitalizeString(property);
}

/**
 * Checks if a property needs to be prefixed
 * @param {String} property - a style property
 */

function isPrefixProperty(property) {
	return requiredProperties.indexOf(property) > -1;
}

/**
 * Generates an array of all relevant properties according to the userAgent
 * @param {string} userAgent - userAgent which gets used to gather information
 */

function generateRequiredProperties() {
	var userAgent = arguments.length <= 0 || arguments[0] === undefined ? ua : arguments[0];

	requiredProperties = [];
	requiredHacks = [];

	if (userAgent) {
		info = (0, _utilsGetBrowserInfo2['default'])(userAgent);
		var data = _data2['default'][info.browser];

		//only generate if there is browser data provided
		if (data) {
			var property = undefined;
			for (property in data) {
				if (data[property] >= info.version) {
					requiredProperties.push(property);
				}
			}

			//add all required hacks for current browser
			var hack = undefined;
			for (hack in _hacks2['default']) {
				var hackData = _hacks2['default'][hack](info);
				if (hackData) {
					requiredHacks.push(hackData);
				}
			}

			generated = true;
			return requiredProperties;
		} else {
			console.warn('Your browser seems to not be supported by inline-style-prefixer.');
			console.warn('Please create an issue at https://github.com/rofrischmann/inline-style-prefixer');
			return false;
		}
	} else {
		console.warn('userAgent needs to be set first. Use `.setUserAgent(userAgent)`');
		return false;
	}
}